vim.cmd([[
  augroup packer_user_config
    autocmd!
    autocmd BufWritePost my_plugins.lua source <afile> | PackerCompile
  augroup end
]])

local fn = vim.fn
local install_path = fn.stdpath('data') .. '/site/pack/packer/start/packer.nvim'
if fn.empty(fn.glob(install_path)) > 0 then
  packer_bootstrap = fn.system({
    'git',
    'clone',
    '--depth',
    '1',
    'https://github.com/wbthomason/packer.nvim',
    install_path,
  })
end
vim.api.nvim_command('packadd packer.nvim')

local function get_setup(name)
  return string.format('require("setup/%s")', name)
end

local packerUtil = require('packer.util')
require('packer').init({
  snapshot_path = packerUtil.join_paths(vim.fn.stdpath('config'), 'snapshots'),
  display = {
    open_fn = require('packer.util').float,
  },
})

return require('packer').startup(function()
  -- Packer can manage itself as an optional plugin
  use({ 'wbthomason/packer.nvim' })
  use({ 'neovim/nvim-lspconfig' })
  use({
    'nvim-treesitter/nvim-treesitter',
    run = ':TSUpdate',
    event = 'BufRead',
    config = get_setup('treesitter'),
  })
  use({
    'nvim-telescope/telescope.nvim',
    requires = {
      { 'nvim-lua/popup.nvim' },
      { 'nvim-lua/plenary.nvim' },
      { 'kyazdani42/nvim-web-devicons' },
      { 'nvim-telescope/telescope-fzf-native.nvim', run = 'make' },
    },
    config = get_setup('telescope'),
  })
  use({
    'sainnhe/gruvbox-material',
    disable = true,
  })
  use({
    'ellisonleao/gruvbox.nvim',
    config = get_setup('gruvbox'),
  })
  use({
    'lukas-reineke/indent-blankline.nvim',
    config = get_setup('indent_blankline'),
  })
  use({ 'nvim-lua/plenary.nvim' })
  use({
    'lewis6991/gitsigns.nvim',
    config = get_setup('gitsigns'),
  })
  use({
    'kyazdani42/nvim-tree.lua',
    requires = 'kyazdani42/nvim-web-devicons',
    config = get_setup('nvim-tree'),
  })
  use({
    'numToStr/Comment.nvim',
    config = get_setup('comment'),
  })
  use({
    'hrsh7th/nvim-cmp',
    event = 'InsertEnter',
    requires = {
      { 'hrsh7th/cmp-buffer', after = 'nvim-cmp' },
      { 'hrsh7th/cmp-nvim-lsp', after = 'nvim-cmp' },
      { 'L3MON4D3/LuaSnip', after = 'nvim-cmp' },
      { 'saadparwaiz1/cmp_luasnip', after = 'nvim-cmp' },
      { 'hrsh7th/cmp-nvim-lua', after = 'nvim-cmp' },
      { 'octaltree/cmp-look', after = 'nvim-cmp' },
      { 'hrsh7th/cmp-path', after = 'nvim-cmp' },
      { 'hrsh7th/cmp-calc', after = 'nvim-cmp' },
      { 'f3fora/cmp-spell', after = 'nvim-cmp' },
      { 'hrsh7th/cmp-emoji', after = 'nvim-cmp' },
      { 'hrsh7th/cmp-cmdline', after = 'nvim-cmp' },
    },
    config = get_setup('cmp'),
  })
  use({ 'L3MON4D3/LuaSnip', event = 'InsertEnter', config = get_setup('luasnip') })
  use({ 'rafamadriz/friendly-snippets' })
  use({ 'onsails/lspkind-nvim' })
  use({ 'Shatur/neovim-cmake', requires = { 'mfussenegger/nvim-dap' }, config = get_setup('cmake') })
  use({
    'hoob3rt/lualine.nvim',
    requires = { 'kyazdani42/nvim-web-devicons', opt = true },
    config = get_setup('lualine'),
  })
  use({
    'ahmedkhalf/project.nvim',
    config = get_setup('project'),
  })
  use({ 'p00f/nvim-ts-rainbow', requires = 'nvim-treesitter/nvim-treesitter', after = 'nvim-treesitter' })
  use({
    'windwp/nvim-autopairs',
    config = get_setup('nvim-autopairs'),
  })
  use({ 'ray-x/lsp_signature.nvim' })
  use({
    's1n7ax/nvim-terminal',
    config = get_setup('terminal'),
  })
  use({
    'williamboman/nvim-lsp-installer',
    requires = {
      'neovim/nvim-lspconfig',
      'p00f/clangd_extensions.nvim',
    },
    config = get_setup('lspinstall'),
  })
  use({
    'jose-elias-alvarez/null-ls.nvim',
    requires = { 'nvim-lua/plenary.nvim' },
  })
  use({
    'danymat/neogen',
    requires = 'nvim-treesitter/nvim-treesitter',
    after = 'nvim-treesitter',
    config = get_setup('neogen'),
  })
  use({ 'abecodes/tabout.nvim' })
  use({ 'stevearc/dressing.nvim' })
  use({
    'nvim-treesitter/nvim-treesitter-textobjects',
    after = 'nvim-treesitter',
    requires = 'nvim-treesitter/nvim-treesitter',
  })
  use({
    'nvim-neo-tree/neo-tree.nvim',
    requires = {
      'nvim-lua/plenary.nvim',
      'kyazdani42/nvim-web-devicons', -- not strictly required, but recommended
      'MunifTanjim/nui.nvim',
    },
    config = get_setup('neo-tree'),
  })
  use({
    'mvllow/modes.nvim',
    config = function()
      vim.opt.cursorline = true
      require('modes').setup({
        focus_only = true,
        line_opacity = 0.2,
      })
    end,
  })
  if packer_bootstrap then
    require('packer').sync()
  end
end)
